%tintern_exact
% Ground-up construction of the Tintern window.
%
% LAST UPDATED by Andy French Augusrt 2023

function tintern_exact

%Window width
a = 1;

%Fontsize for graphs
fsize = 18;

%Linewidth
L = 2;

%Patch windows?
patch_or_circle = 'patch';

%

%Set up figure
fig = figure('name','tintern window','color',[1 1 1]);
ax = axes('Nextplot','add'); axis equal; xlim([-0.2*a,1.2*a]); ylim([-a,a]);
grid on; box on; set(gca,'fontsize',fsize);

%Plot base and vertical columns
plot( [0,a],-0.75*a*[1,1], 'b','linewidth',L );
for n=1:8
    plot( (n-1)*(a/7)*[1,1],[-0.75*a,0],'b','linewidth',L );
end

%Plot all arches
for n=1:7
    w = a - (n-1)*a/7; x0 = (n-1)*a/7;
    arch_plotter( a, w, x0, L,[0,0,1] );
    arch_plotter( a, w, 0, L,[0,0,1] );
end

%Calculate radius of smaller tangent circle
R = a*sqrt( ( 5/sqrt(38) - 5/7 )^2 + ( sqrt(494)/38 - sqrt(13)/7 )^2 );

%Plot smaller tangent circles
circle_plotter( 5*a/7, a*sqrt(13)/7, R, patch_or_circle, L, [0,0,1], [1,1,1] )
circle_plotter( 2*a/7, a*sqrt(13)/7, R, patch_or_circle, L, [0,0,1], [1,1,1] )

%Calculate centre height of top circle vi a numeric method
h = linspace( a*sqrt(13)/7, a*sqrt(3)/2, 1000 );
f = ( a - 0.5*a*sqrt( 1 + 4*(h.^2)/(a^2) ) + R ) .^2 - 9*(a^2)/196 - ( h - a*sqrt(13)/7 ).^2;
h = rootfindergeneral(h,f);

%Calculate radius of top circle given h
r = a - 0.5*a*sqrt( 1 + 4*(h.^2)/(a^2) );

%What is the ratio r/R ?
disp( ['r/R = ',num2str(r/R,7)] );

%Plot top circle
circle_plotter( a/2, h, r, patch_or_circle, L, [0,0,1], [1,1,1] )

%Save PNG and close
title(['Tintern window. a=',num2str(a),', h=',num2str(h,3),...
        ', r=',num2str(r,3),', R=',num2str(R,3)])
print(gcf,'tintern_exact.png','-r300','-dpng'); close(gcf);

%%

%Arch plotter. Plots an arch formed from an arc of a circle of radius a
%and arch width w, starting at x=x0. L is the linewidth, RGB is the line
%colour
function arch_plotter( a,w,x0,L,RGB )
A = a - w/2; B = sqrt( a^2 - A^2 );
t = linspace(0,atan(B/A),1000);
d = a-w-x0;
xR = a*cos(t); xL = 2*A-xR; 
xR = xR - d; xL = xL - d;
y = a*sin(t);
plot( xR,y, 'color',RGB,'linewidth',L );
plot( xL,y, 'color',RGB,'linewidth',L );

%%

%Circle plotter
function circle_plotter( xc, yc, r, patch_or_circle, L, RGB_line, RGB_face )
t = linspace(0,2*pi,1000);
x = r*cos(t) + xc; y = r*sin(t) + yc;
if strcmp( patch_or_circle, 'patch' )
    patch(x,y,ones(1,length(x)), RGB_face,'edgecolor',RGB_line,'linewidth',L );
else
    plot(x,y, 'color',RGB_line,'linewidth',L );
end

%%

%Find root(s) of function f(x)
function xroot = rootfindergeneral(x,f)
xroot = [];
for n=1:length(x)-1
    %Check if f(x) has crossed zero
    if ( ( f(n+1)>=0 ) && ( f(n) <=0 ) ) || ( ( f(n+1)<=0 ) && ( f(n) >=0 ) )
        xroot = [xroot,0.5*( x(n) + x(n+1) )];
    end
end

%Plot solution method
figure; set(gca, 'fontsize', 18); plot( x,real(f),'linewidth',2);  
hold on; grid on; xlimits = get(gca,'xlim'); ylimits = get(gca,'ylim');
plot( [xroot,xroot],ylimits ); plot( xlimits,[0,0] );
title( ['x root = ',num2str(xroot) ] ); xlabel('x'); ylabel('f(x)');
print( gcf, 'rootfinder.png','-dpng','-r300' ); close(gcf);

%End of code
